@@ -2,11 +2,12 @@ import os
 import sys
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import TYPE_CHECKING, Dict, Iterable, Iterator, Optional, Set, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Iterator, Optional, Sequence, Set, Tuple, Type, Union
 
 from django.core.exceptions import FieldError
 from django.db import models
 from django.db.models.base import Model
+from django.db.models.expressions import Expression
 from django.db.models.fields import AutoField, CharField, Field
 from django.db.models.fields.related import ForeignKey, RelatedField
 from django.db.models.fields.reverse_related import ForeignObjectRel
@@ -14,12 +15,15 @@ from django.db.models.lookups import Exact
 from django.db.models.sql.query import Query
 from django.utils.functional import cached_property
 from mypy.checker import TypeChecker
+from mypy.nodes import TypeInfo
 from mypy.plugin import MethodContext
 from mypy.types import AnyType, Instance
 from mypy.types import Type as MypyType
 from mypy.types import TypeOfAny, UnionType
+from typing_extensions import Literal
 
 from mypy_django_plugin.lib import fullnames, helpers
+from mypy_django_plugin.lib.fullnames import WITH_ANNOTATIONS_FULLNAME
 
 try:
     from django.contrib.postgres.fields import ArrayField
@@ -32,10 +36,11 @@ except ImportError:
 if TYPE_CHECKING:
     from django.apps.registry import Apps  # noqa: F401
     from django.conf import LazySettings  # noqa: F401
+    from django.contrib.contenttypes.fields import GenericForeignKey
 
 
 @contextmanager
-def temp_environ():
+def temp_environ() -> Iterator[None]:
     """Allow the ability to set os.environ temporarily"""
     environ = dict(os.environ)
     try:
@@ -52,14 +57,6 @@ def initialize_django(settings_module: str) -> Tuple["Apps", "LazySettings"]:
         # add current directory to sys.path
         sys.path.append(os.getcwd())
 
-        def noop_class_getitem(cls, key):
-            return cls
-
-        from django.db import models
-
-        models.QuerySet.__class_getitem__ = classmethod(noop_class_getitem)  # type: ignore
-        models.Manager.__class_getitem__ = classmethod(noop_class_getitem)  # type: ignore
-
         from django.apps import apps
         from django.conf import settings
 
@@ -67,12 +64,12 @@ def initialize_django(settings_module: str) -> Tuple["Apps", "LazySettings"]:
         apps.get_swappable_settings_name.cache_clear()  # type: ignore
 
         if not settings.configured:
-            settings._setup()
+            settings._setup()  # type: ignore
 
         apps.populate(settings.INSTALLED_APPS)
 
-    assert apps.apps_ready
-    assert settings.configured
+    assert apps.apps_ready, "Apps are not ready"
+    assert settings.configured, "Settings are not configured"
 
     return apps, settings
 
@@ -91,10 +88,7 @@ class DjangoContext:
 
     @cached_property
     def model_modules(self) -> Dict[str, Set[Type[Model]]]:
-        """ All modules that contain Django models. """
-        if self.apps_registry is None:
-            return {}
-
+        """All modules that contain Django models."""
         modules: Dict[str, Set[Type[Model]]] = defaultdict(set)
         for concrete_model_cls in self.apps_registry.get_models():
             modules[concrete_model_cls.__module__].add(concrete_model_cls)
@@ -105,14 +99,23 @@ class DjangoContext:
         return modules
 
     def get_model_class_by_fullname(self, fullname: str) -> Optional[Type[Model]]:
-        # Returns None if Model is abstract
+        """Returns None if Model is abstract"""
+        annotated_prefix = WITH_ANNOTATIONS_FULLNAME + "["
+        if fullname.startswith(annotated_prefix):
+            # For our "annotated models", extract the original model fullname
+            fullname = fullname[len(annotated_prefix) :].rstrip("]")
+            if "," in fullname:
+                # Remove second type arg, which might be present
+                fullname = fullname[: fullname.index(",")]
+            fullname = fullname.replace("__", ".")
+
         module, _, model_cls_name = fullname.rpartition(".")
         for model_cls in self.model_modules.get(module, set()):
             if model_cls.__name__ == model_cls_name:
                 return model_cls
         return None
 
-    def get_model_fields(self, model_cls: Type[Model]) -> Iterator[Field]:
+    def get_model_fields(self, model_cls: Type[Model]) -> Iterator["Field[Any, Any]"]:
         for field in model_cls._meta.get_fields():
             if isinstance(field, Field):
                 yield field
@@ -122,7 +125,9 @@ class DjangoContext:
             if isinstance(field, ForeignObjectRel):
                 yield field
 
-    def get_field_lookup_exact_type(self, api: TypeChecker, field: Union[Field, ForeignObjectRel]) -> MypyType:
+    def get_field_lookup_exact_type(
+        self, api: TypeChecker, field: Union["Field[Any, Any]", ForeignObjectRel]
+    ) -> MypyType:
         if isinstance(field, (RelatedField, ForeignObjectRel)):
             related_model_cls = field.related_model
             primary_key_field = self.get_primary_key_field(related_model_cls)
@@ -140,7 +145,21 @@ class DjangoContext:
             return AnyType(TypeOfAny.explicit)
         return helpers.get_private_descriptor_type(field_info, "_pyi_lookup_exact_type", is_nullable=field.null)
 
-    def get_primary_key_field(self, model_cls: Type[Model]) -> Field:
+    def get_related_target_field(
+        self, related_model_cls: Type[Model], field: "ForeignKey[Any, Any]"
+    ) -> "Optional[Field[Any, Any]]":
+        # ForeginKey only supports one `to_fields` item (ForeignObject supports many)
+        assert len(field.to_fields) == 1
+        to_field_name = field.to_fields[0]
+        if to_field_name:
+            rel_field = related_model_cls._meta.get_field(to_field_name)
+            if not isinstance(rel_field, Field):
+                return None  # Not supported
+            return rel_field
+        else:
+            return self.get_primary_key_field(related_model_cls)
+
+    def get_primary_key_field(self, model_cls: Type[Model]) -> "Field[Any, Any]":
         for field in model_cls._meta.get_fields():
             if isinstance(field, Field):
                 if field.primary_key:
@@ -159,10 +178,29 @@ class DjangoContext:
             field_set_type = self.get_field_set_type(api, primary_key_field, method=method)
             expected_types["pk"] = field_set_type
 
+        def get_field_set_type_from_model_type_info(info: Optional[TypeInfo], field_name: str) -> Optional[MypyType]:
+            if info is None:
+                return None
+            field_node = info.names.get(field_name)
+            if field_node is None or not isinstance(field_node.type, Instance):
+                return None
+            elif not field_node.type.args:
+                # Field declares a set and a get type arg. Fallback to `None` when we can't find any args
+                return None
+
+            set_type = field_node.type.args[0]
+            return set_type
+
+        model_info = helpers.lookup_class_typeinfo(api, model_cls)
         for field in model_cls._meta.get_fields():
             if isinstance(field, Field):
                 field_name = field.attname
-                field_set_type = self.get_field_set_type(api, field, method=method)
+                # Try to retrieve set type from a model's TypeInfo object and fallback to retrieving it manually
+                # from django-stubs own declaration. This is to align with the setter types declared for
+                # assignment.
+                field_set_type = get_field_set_type_from_model_type_info(
+                    model_info, field_name
+                ) or self.get_field_set_type(api, field, method=method)
                 expected_types[field_name] = field_set_type
 
                 if isinstance(field, ForeignKey):
@@ -198,7 +236,12 @@ class DjangoContext:
                 # it's generic, so cannot set specific model
                 field_name = field.name
                 gfk_info = helpers.lookup_class_typeinfo(api, field.__class__)
-                gfk_set_type = helpers.get_private_descriptor_type(gfk_info, "_pyi_private_set_type", is_nullable=True)
+                if gfk_info is None:
+                    gfk_set_type: MypyType = AnyType(TypeOfAny.unannotated)
+                else:
+                    gfk_set_type = helpers.get_private_descriptor_type(
+                        gfk_info, "_pyi_private_set_type", is_nullable=True
+                    )
                 expected_types[field_name] = gfk_set_type
 
         return expected_types
@@ -219,11 +262,14 @@ class DjangoContext:
     def all_registered_model_class_fullnames(self) -> Set[str]:
         return {helpers.get_class_fullname(cls) for cls in self.all_registered_model_classes}
 
-    def get_attname(self, field: Field) -> str:
+    def get_attname(self, field: "Field[Any, Any]") -> str:
         attname = field.attname
         return attname
 
-    def get_field_nullability(self, field: Union[Field, ForeignObjectRel], method: Optional[str]) -> bool:
+    def get_field_nullability(self, field: Union["Field[Any, Any]", ForeignObjectRel], method: Optional[str]) -> bool:
+        if method in ("values", "values_list"):
+            return field.null
+
         nullable = field.null
         if not nullable and isinstance(field, CharField) and field.blank:
             return True
@@ -237,8 +283,10 @@ class DjangoContext:
             return True
         return nullable
 
-    def get_field_set_type(self, api: TypeChecker, field: Union[Field, ForeignObjectRel], *, method: str) -> MypyType:
-        """ Get a type of __set__ for this specific Django field. """
+    def get_field_set_type(
+        self, api: TypeChecker, field: Union["Field[Any, Any]", ForeignObjectRel], *, method: str
+    ) -> MypyType:
+        """Get a type of __set__ for this specific Django field."""
         target_field = field
         if isinstance(field, ForeignKey):
             target_field = field.target_field
@@ -255,8 +303,10 @@ class DjangoContext:
             field_set_type = helpers.convert_any_to_type(field_set_type, argument_field_type)
         return field_set_type
 
-    def get_field_get_type(self, api: TypeChecker, field: Union[Field, ForeignObjectRel], *, method: str) -> MypyType:
-        """ Get a type of __get__ for this specific Django field. """
+    def get_field_get_type(
+        self, api: TypeChecker, field: Union["Field[Any, Any]", ForeignObjectRel], *, method: str
+    ) -> MypyType:
+        """Get a type of __get__ for this specific Django field."""
         field_info = helpers.lookup_class_typeinfo(api, field.__class__)
         if field_info is None:
             return AnyType(TypeOfAny.unannotated)
@@ -267,7 +317,7 @@ class DjangoContext:
             if related_model_cls is None:
                 return AnyType(TypeOfAny.from_error)
 
-            if method == "values":
+            if method in ("values", "values_list"):
                 primary_key_field = self.get_primary_key_field(related_model_cls)
                 return self.get_field_get_type(api, primary_key_field, method=method)
 
@@ -279,14 +329,16 @@ class DjangoContext:
         else:
             return helpers.get_private_descriptor_type(field_info, "_pyi_private_get_type", is_nullable=is_nullable)
 
-    def get_field_related_model_cls(self, field: Union[RelatedField, ForeignObjectRel]) -> Optional[Type[Model]]:
+    def get_field_related_model_cls(
+        self, field: Union["RelatedField[Any, Any]", ForeignObjectRel]
+    ) -> Optional[Type[Model]]:
         if isinstance(field, RelatedField):
             related_model_cls = field.remote_field.model
         else:
             related_model_cls = field.field.model
 
         if isinstance(related_model_cls, str):
-            if related_model_cls == "self":
+            if related_model_cls == "self":  # type: ignore
                 # same model
                 related_model_cls = field.model
             elif "." not in related_model_cls:
@@ -294,15 +346,18 @@ class DjangoContext:
                 related_model_fullname = field.model.__module__ + "." + related_model_cls
                 related_model_cls = self.get_model_class_by_fullname(related_model_fullname)
             else:
-                related_model_cls = self.apps_registry.get_model(related_model_cls)
+                try:
+                    related_model_cls = self.apps_registry.get_model(related_model_cls)
+                except LookupError:
+                    return None
 
         return related_model_cls
 
     def _resolve_field_from_parts(
         self, field_parts: Iterable[str], model_cls: Type[Model]
-    ) -> Union[Field, ForeignObjectRel]:
+    ) -> Union["Field[Any, Any]", ForeignObjectRel]:
         currently_observed_model = model_cls
-        field = None
+        field: Union["Field[Any, Any]", ForeignObjectRel, GenericForeignKey, None] = None
         for field_part in field_parts:
             if field_part == "pk":
                 field = self.get_primary_key_field(currently_observed_model)
@@ -318,27 +373,44 @@ class DjangoContext:
             if isinstance(field, ForeignObjectRel):
                 currently_observed_model = field.related_model
 
-        assert field is not None
+        # Guaranteed by `query.solve_lookup_type` before.
+        assert isinstance(field, (Field, ForeignObjectRel))
         return field
 
-    def resolve_lookup_into_field(self, model_cls: Type[Model], lookup: str) -> Union[Field, ForeignObjectRel]:
+    def solve_lookup_type(
+        self, model_cls: Type[Model], lookup: str
+    ) -> Optional[Tuple[Sequence[str], Sequence[str], Union[Expression, Literal[False]]]]:
         query = Query(model_cls)
-        lookup_parts, field_parts, is_expression = query.solve_lookup_type(lookup)
+        if (lookup == "pk" or lookup.startswith("pk__")) and query.get_meta().pk is None:
+            # Primary key lookup when no primary key field is found, model is presumably
+            # abstract and we can't say anything about 'pk'.
+            return None
+        return query.solve_lookup_type(lookup)
+
+    def resolve_lookup_into_field(
+        self, model_cls: Type[Model], lookup: str
+    ) -> Union["Field[Any, Any]", ForeignObjectRel, None]:
+        solved_lookup = self.solve_lookup_type(model_cls, lookup)
+        if solved_lookup is None:
+            return None
+        lookup_parts, field_parts, is_expression = solved_lookup
         if lookup_parts:
             raise LookupsAreUnsupported()
-
         return self._resolve_field_from_parts(field_parts, model_cls)
 
     def resolve_lookup_expected_type(self, ctx: MethodContext, model_cls: Type[Model], lookup: str) -> MypyType:
-        query = Query(model_cls)
         try:
-            lookup_parts, field_parts, is_expression = query.solve_lookup_type(lookup)
-            if is_expression:
-                return AnyType(TypeOfAny.explicit)
+            solved_lookup = self.solve_lookup_type(model_cls, lookup)
         except FieldError as exc:
             ctx.api.fail(exc.args[0], ctx.context)
             return AnyType(TypeOfAny.from_error)
 
+        if solved_lookup is None:
+            return AnyType(TypeOfAny.implementation_artifact)
+        lookup_parts, field_parts, is_expression = solved_lookup
+        if is_expression:
+            return AnyType(TypeOfAny.explicit)
+
         field = self._resolve_field_from_parts(field_parts, model_cls)
 
         lookup_cls = None
